#include <queue>
#include <iostream>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include "AppExitStates.cpp"

void Deinicialization()
{
	if (TTF_WasInit())
		TTF_Quit();
	IMG_Quit();
	if (SDL_WasInit(SDL_INIT_VIDEO))
		SDL_Quit();
}

int inicialization()
{
	if (SDL_Init(SDL_INIT_VIDEO) < 0)
	{
		SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION, "SDL was unable to inicializate. %s", SDL_GetError());
		SDL_ClearError();
		Deinicialization();
		return AppExitStates::InicializationFailed;
	};
	if (IMG_Init(IMG_INIT_PNG) == 0)
	{
		SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION, "IMG was unable to inicializate.");
		Deinicialization();
		return AppExitStates::InicializationFailed;
	};
	if (TTF_Init() < 0)
	{
		SDL_LogCritical(SDL_LOG_CATEGORY_APPLICATION, "TTF was unable to inicializate. %s", TTF_GetError());
		Deinicialization();
		return AppExitStates::InicializationFailed;
	};
	return 0;
}

int SDL_main(int, char**)
{
	int Init = inicialization();
	if (Init)
		return Init;

	//vytvoen okna
	SDL_Window* MainWindow = SDL_CreateWindow("Errory a logovani", //jmno okna
		SDL_WINDOWPOS_CENTERED, //okno bude vytvoeno ve stedu obrazovky
		SDL_WINDOWPOS_CENTERED,
		800, 600, NULL); //ka, vka a flagy okna (ty nepotebujeme)

	if (MainWindow == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Windows was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfBasicObjectsFailed;
	}

	//vytvoen rendereru
	SDL_Renderer* renderer = SDL_CreateRenderer(MainWindow, -1, NULL);
	if (renderer == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Renderer was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfBasicObjectsFailed;
	}

	//vytvoen 32 bitov surface								      0xF800, 0x7C0, 0x3E, 1);
	SDL_Surface* SurfaceWith32Transition = SDL_CreateRGBSurface(NULL, 256, 256, 32, 0xFF0000, 0x00FF00, 0x0000FF, 0);
	if (SurfaceWith32Transition == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Surface was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfRenderingObjectsFailed;
	}
	Uint8* pixels = (Uint8*)SurfaceWith32Transition->pixels;
	SDL_PixelFormat* format = SurfaceWith32Transition->format;
	for (int a = 0; a < 256; a++)
		for (int b = 0; b < 256; b++)
		{
			Uint8* pixel = pixels + a*SurfaceWith32Transition->pitch + b*format->BytesPerPixel;
			*(Uint32*)pixel = a << format->Rshift | b << format->Gshift;
		}
	SDL_Texture* TextureWith32Transition = SDL_CreateTextureFromSurface(renderer, SurfaceWith32Transition);
	if (TextureWith32Transition == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Texture was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfRenderingObjectsFailed;
	}
	SDL_Rect* Transition32Rect = new SDL_Rect;
	SDL_FreeSurface(SurfaceWith32Transition);
	Transition32Rect->x = Transition32Rect->y = 0;
	Transition32Rect->w = Transition32Rect->h = 256;


	//vytvoen 16 bitov surface								      0xF800, 0x7C0, 0x3E, 1);
	SDL_Surface* SurfaceWith16Transition = SDL_CreateRGBSurface(NULL, 256, 256, 16, 0xF00, 0x0F0, 0x00F, 0xF000);
	if (SurfaceWith16Transition == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Surface was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfRenderingObjectsFailed;
	}
	pixels = (Uint8*)SurfaceWith16Transition->pixels;
	format = SurfaceWith16Transition->format;
	for (int a = 0; a < 256; a++)
		for (int b = 0; b < 256; b++)
		{
			int GreenColor = b >> format->Gloss;
			int RedColor = a >> format->Rloss;
			Uint8* pixel = pixels + a*SurfaceWith16Transition->pitch + b*format->BytesPerPixel;
			*(Uint16*)pixel = RedColor << format->Rshift | GreenColor << format->Gshift | 0xF << format->Ashift;
		}
	SDL_Texture* TextureWith16Transition = SDL_CreateTextureFromSurface(renderer, SurfaceWith16Transition);
	if (TextureWith16Transition == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Texture was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfRenderingObjectsFailed;
	}
	SDL_FreeSurface(SurfaceWith16Transition);
	SDL_Rect* Transition16Rect = new SDL_Rect;
	Transition16Rect->x = 0;
	Transition16Rect->y = Transition16Rect->w = Transition16Rect->h = 256;


	//animovana Surface
	SDL_Surface* AnimatedSurface = SDL_CreateRGBSurface(NULL, 256, 100, 32, 0xFF0000, 0x00FF00, 0x0000FF, 0);
	if (AnimatedSurface == NULL)
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Surface was unable to create. %s", SDL_GetError());
		SDL_ClearError();
		return AppExitStates::CreatingOfRenderingObjectsFailed;
	}
	pixels = (Uint8*)AnimatedSurface->pixels;
	format = AnimatedSurface->format;
	for (int a = 0; a < 255; a++)
		for (int b = 0; b < 100; b++)
		{
			Uint8* pixel = pixels + b*AnimatedSurface->pitch + a*format->BytesPerPixel;
			*(Uint32*)pixel = a << format->Rshift;
		}
	SDL_Rect* AnimatedRect = new SDL_Rect;
	AnimatedRect->x = 300;
	AnimatedRect->y = 200;
	AnimatedRect->w = 256;
	AnimatedRect->h = 100;

	//demo logovn
	SDL_LogSetAllPriority(SDL_LOG_PRIORITY_VERBOSE); //od kter priority chceme zprvy zobrazit
	SDL_LogVerbose(SDL_LOG_CATEGORY_APPLICATION, "Verbose application log");
	SDL_LogDebug(SDL_LOG_CATEGORY_ASSERT, "Debug assert log");
	SDL_LogInfo(SDL_LOG_CATEGORY_AUDIO,"Info audio log");
	SDL_LogWarn(SDL_LOG_CATEGORY_ERROR,"Warn error log");
	SDL_LogError(SDL_LOG_CATEGORY_INPUT, "Error input log");
	SDL_LogCritical(SDL_LOG_CATEGORY_RENDER, "Critical render log");

	bool End = false;								//promnn, kter d hlavn smyku
	SDL_Event* event = new SDL_Event;				//struktura, kter uchovv udlosti
	while (!End)									//zatek hlavn smyky
	{
		while (SDL_PollEvent(event))				//naten udlosti z fronty
		{
			if (event->type == SDL_QUIT)				//manipulace s udlost
				End = true;							//ukonen aplikace
		}

		//update prechodu
		std::queue<int> QueueWithColors;
		for (int a = 1; a < 256; a++)
		{
			Uint8* pixel = pixels + a*format->BytesPerPixel;
			QueueWithColors.push(*(Uint32*)pixel);
		}
		QueueWithColors.push(*(Uint32*)pixels);
		for (int a = 0; a < 256; a++)
		{
			int ColorToUse = QueueWithColors.front();
			for (int b = 0; b < 100; b++)
			{
				Uint8* pixel = pixels + b*AnimatedSurface->pitch + a*format->BytesPerPixel;
				*(Uint32*)pixel = ColorToUse;
			}
			QueueWithColors.pop();
		}





		if (SDL_SetRenderDrawColor(renderer, 0, 0, 255, 0) < 0)	//nastaven barvy, kterou se bude vykreslovat pozad
		{
			SDL_LogWarn(SDL_LOG_CATEGORY_RENDER, "Failed to set renderer draw color. %s", SDL_GetError());
			SDL_ClearError();
		};
		if (SDL_RenderClear(renderer) < 0)								//vykreslen pozad
		{
			SDL_LogWarn(SDL_LOG_CATEGORY_RENDER, "Failed to clear renderer. %s", SDL_GetError());
			SDL_ClearError();
		};

		//vykreslen pechodu
		if (SDL_RenderCopy(renderer, TextureWith32Transition, NULL, Transition32Rect) < 0 ||
			SDL_RenderCopy(renderer, TextureWith16Transition, NULL, Transition16Rect) < 0)
		{
			SDL_LogWarn(SDL_LOG_CATEGORY_RENDER, "Failed to render textures. %s", SDL_GetError());
			SDL_ClearError();
		}

		//vykresleni animovaneho prechodu
		SDL_Texture* AnimatedTexture = SDL_CreateTextureFromSurface(renderer, AnimatedSurface);
		if (AnimatedTexture == NULL)
		{
			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Texture was unable to create. %s", SDL_GetError());
			SDL_ClearError();
			return AppExitStates::CreatingOfRenderingObjectsFailed;
		}

		if(SDL_RenderCopy(renderer, AnimatedTexture, NULL, AnimatedRect)<0)
		{
			SDL_LogWarn(SDL_LOG_CATEGORY_RENDER, "Failed to render texture. %s", SDL_GetError());
			SDL_ClearError();
		};

		SDL_DestroyTexture(AnimatedTexture);

		SDL_RenderPresent(renderer);


	}

	//nen poteba mazat jednotliv objekty, po ukonen programu voln msto automaticky promae OS

	Deinicialization();
	return AppExitStates::Success;
}